package PureMessage::Policy::RemoteScan::Config;

use strict;
use PureMessage::Config qw(expand_filename);
use base qw(PureMessage::Config::Validator Exporter);

our @EXPORT_OK = '%config';
our %config;

my %nodes = (
    'profile#*.reason'                  => ['#str',  undef],
    'profile#*.public_review_addr'      => ['#str',  undef],
    'profile#*.confidential_review_addr'=> ['#str',  undef],
    'profile#*.deny_template'           => ['#file', undef],
    
    'profile#*.logfile'                 => ['#file', 'var/log/remotescan_log'],
    'profile#*.journal_directory'       => ['#file', 'var/journal/remotescan'],
    'profile#*.release_search_max'      => ['#int', 5],

    'profile#*.scan_multipart_preamble' => ['#bool', 0],
    'profile#*.scan_message_rfc822'     => ['#bool', 0],
    'profile#*.scan_inline_plaintext'   => ['#bool', 0],
    'profile#*.port'                    => ['#port', undef],
    'profile#*.timeout_connect'         => ['#duration', '15s'],
    'profile#*.timeout_command'         => ['#duration', '15s'],
    'profile#*.code#*.match'            => ['#str',  undef],
    'profile#*.code#*.select'           => ['#bool', undef],
    'profile#*.code#*.return'           => ['#bool', undef],
    'profile#*.code#*.priority'         => ['#int',  0],
);

refresh();

sub new {
    my $self = shift;
    require PureMessage::Config::Validator;
    my $p = ActiveState::Config->new(strict => 1,
				     allow_includes => 1);
    PureMessage::Config::Validator->define_core_validators($p);
    PureMessage::Policy::RemoteScan::Config->validate($p);
    $p->parse_file(expand_filename("etc/remotescan.conf"), @_);
    $p;
}

sub refresh {
    %config = new(undef)->as_hash;
}

sub d_cclim {
    require PureMessage::_sys;
    return PureMessage::_sys::available_cpus() * 2;
}

sub validate {
    my ($pkg, $p) = @_;

    for my $key (keys %nodes) {
	my ($t, $d) = @{$nodes{$key}};
	$p->typedecl($key, $t) if defined $t;
	$p->default($key, $d)  if defined $d;
    }

    $pkg->include_sub_validators($p);
}

1;
