#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w

eval 'exec /tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w -S $0 ${1+"$@"}'
    if 0; # not running under some shell

use strict;
use pmx_user;
use PureMessage::Config qw(expand_tmpl_filename);
use PureMessage::Policy::RemoteScan::Config qw(%config);
use PureMessage::Logger;
use PureMessage::Quarantine;
use PureMessage::QMan ();
use PureMessage::Queue qw(queue_mail);
use PureMessage::Template qw(expand_template);
use Digest::MD5 qw(md5_hex);

my $logger = PureMessage::Logger->new(ident => 'release-review', 
                                  log_to => $config{profile}{common}{logfile});
my $q = PureMessage::Quarantine->new;
$q->message_writable(1);

my %CMD = (
    approve => \&approve,
    deny    => \&deny,
);

# We don't care who the mail is from -- if they know the quarantine secret and
# the queue IDs, we'll release them.  
# This makes me afraid - klez
do { local $/ = ""; <> }; # skip headers
while (<>) {
    chomp;
    my ($cmd, $qid, $hash, @kruft) = split ' ', $_, 4;
    next if @kruft;
    next unless $CMD{$cmd};

    # Validate the $hash:
    next unless check_hash($cmd, $qid, $hash);

    my $mset = $q->message_set(
        folder => "ReviewPending",
        fields => [qw(id m_reason)],
        where  => qq#m_queue_id = "$qid"#,
    );
    $mset->execute;
    my $i = 0;
    while (my $row = $mset->fetch($i)) {
        my ($id, $reason) = $row->fields;
        eval { $CMD{$cmd}->($id, $reason, $cmd, $qid, $hash) };
        $logger->log("WARNING", "error trying to '$cmd' $qid: $@") if $@;
        ++$i;
    }
    $logger->log("WARNING", "could not '$cmd' $qid: message not found")
        unless $i;
}

# This doesn't need to be a function, but it could be abstracted out to
# another module at some point...
sub check_hash {
    my ($cmd, $qid, $hash) = @_;
    my $secret = $PureMessage::Config::config{quarantine_secret};
    my $calc = md5_hex("$qid $secret");
    my $ok = $calc eq $hash;
    $logger->log(
        "WARNING",
        "can't '$cmd' $qid: hash '$hash' != calculated '$calc'"
    ) unless $ok;
    return $ok;
}

sub approve {
    my ($id, $reason, $cmd, $qid, $hash) = @_;
    my $msg = $q->message($id);

    # Now get the *real* MessageFile object 
    # (required when we're in pmdb mode)
    my $filename = $q->id_to_filename($id);
    my $msgf = PureMessage::MessageFile->new($filename, writable => 1);

    if ($msgf->minfo_value('Review-Pending') && 
        $msgf->minfo_value('Review-Pending') eq 'true')
    {
        PureMessage::QMan::approve(
                $msg,
                trash => 1,
                logger => $logger,
                quarantine => $q,
                );
        $logger->log("INFO", "Approved $id. (queue_id $qid)");
    }
    else {
        $logger->log("WARNING", "Attempted to approve $id, which is not marked as ".
                "Review-Pending. (queue_id $qid)");
    }
}

sub deny {
    my ($id, $reason, $cmd, $qid, $hash) = @_;
    my $msg = $q->message($id);
    my $tmpl;
    for my $profile (keys %{$config{profile}}) {
        my $def = $config{profile}{$profile};
        if ($reason eq $def->{reason}) {
            $tmpl = expand_tmpl_filename($def->{deny_template});
            last;
        }
    }
    die "Can't deny message $id (queue_id $qid): can't find profile $reason"
        unless $tmpl;
    my $content = do {
        open (my $FH, $tmpl) or die "can't open $tmpl: $!";
        local $/;
        <$FH>;
    };
    queue_mail(
        from => $PureMessage::Config::config{admin},
        to   => [$msg->minfo_value("Envelope-To")],
        message => expand_template(
            $content,
            vars => {
                to       => $msg->header("To") || "(none)",
                from     => $msg->header("From") || "(none)",
                subject  => $msg->header("Subject") || "(none)",
                action   => $cmd,
                queue_id => $qid,
            },
        ),
    );
}
